const rssXMLHeader = '<?xml version="1.0" encoding="UTF-8"?>\n';
const ns =
{
	rdf: "http://www.w3.org/1999/02/22-rdf-syntax-ns#",
	dc: "http://purl.org/dc/elements/1.1/",
	content: "http://purl.org/rss/1.0/modules/content/",
	rss1:"http://purl.org/rss/1.0/"
}
const rssType_all = 0;
const rssType_selection = 1;


function StringOutputStream() {this.buf = '';}
StringOutputStream.prototype = {
	QueryInterface: function(iid)
	{
		if (!iid.equals(Components.interfaces.nsISupports) &&
			!iid.equals(Components.interfaces.nsIOutputStream))
		throw Components.results.NS_ERROR_NO_INTERFACE;
		return this;
	},
	close : function () {},
	flush : function () {},
	isNonBlocking : function () { return false; },
	write : function (buf, count)
	{
		if (count < 0 || count > buf.length)
		{
			this.buf += buf;
			return buf.length;
		}
		this.buf += buf.substr(0,count);
		return count;
	},
	writeFrom : function(fromStream, count) { throw Components.results.NS_ERROR_NOT_IMPLEMENTED; },
	writeSegments : function(reader, closure , count) { throw Components.results.NS_ERROR_NOT_IMPLEMENTED; }
};


/**
 * Get the text in the clipboard w/ the mime type text/unicode
 * @return string String of data if successful, FALSE if failure
 */
function getClipboardText()
{
	var clip = Components.classes["@mozilla.org/widget/clipboard;1"].createInstance(Components.interfaces.nsIClipboard);
	if (!clip) return false;

	var trans = Components.classes["@mozilla.org/widget/transferable;1"].createInstance(Components.interfaces.nsITransferable);
	if (!trans) return false;
	trans.addDataFlavor("text/unicode");

	clip.getData(trans, clip.kGlobalClipboard);

	var str=new Object();
	var strLength=new Object();

	trans.getTransferData("text/unicode", str, strLength);

	if (str) str=str.value.QueryInterface(Components.interfaces.nsISupportsString);
	if (str) pastetext=str.data.substring(0, strLength.value / 2);

	return pastetext;
}


/**
 * Parse an item from the DOM Element "item" - called by parseDoc
 * @param Array_of_items_elements items
 */
function parseItems(items)
{
	var src = items.length ? items : [items];
	var data = [];
	var item, itemSrc, count2, j, pDate;
	try
	{
		var count = src.length;
		for (var i=0; i<count; i++)
		{
			item = new RSSItem();
			itemSrc = src[i].childNodes;
			if (src[i].hasAttributeNS(ns.rdf, "about"))
				item.guid = src[i].getAttributeNS(ns.rdf, "about");
			count2 = itemSrc.length;
			for (j=0; j<count2; j++)
			{
				switch (itemSrc[j].localName)
				{
					case "title":
					case "link":
					case "comments":
						try
						{
							item[itemSrc[j].nodeName] = itemSrc[j].firstChild.data;
						}
						catch (e) {}
						break;

					case "encoded":
						if (itemSrc[j].namespaceURI != ns.content) break;
					case "description":
						try
						{
							item.desc = itemSrc[j].firstChild.data;
						}
						catch (e) {}
						break;

					case "creator":
						if (itemSrc[j].namespaceURI != ns.dc) break;
					case "author":
						try
						{
							item.author = itemSrc[j].firstChild.data;
						}
						catch (e) {}
						break;

					case "subject":
						if (itemSrc[j].namespaceURI != ns.dc) break;
					case "category":
						try
						{
							item.category = itemSrc[j].firstChild.data;
							// below is only for RSS 0,9/2.0 but not DC
							if (itemSrc[j].localName == "category" && itemSrc[j].hasAttribute("domain")) item.domain = itemSrc[j].getAttribute("domain");
						}
						catch (e) {}
						break;

					case "enclosure":
						for (var key in item.enclosure)
							if (itemSrc[j].hasAttribute(key)) item.enclosure[key] = itemSrc[j].getAttribute(key);
						break;

					case "guid":
						try
						{
							item.guid = itemSrc[j].firstChild.data;
							item.isPermaLink = !itemSrc[j].hasAttribute("isPermaLink") || itemSrc[j].getAttribute("isPermaLink") != "false";
						}
						catch (e) {}
						break;

					case "date":
						if (itemSrc[j].namespaceURI != ns.dc) break;
						try
						{
							pDate = itemSrc[j].firstChild.data.match(/(\d{4})-(\d\d)-(\d\d)(?:T(\d\d:\d\d(?::\d\d)?)(?:\.\d+)?(Z|[+-]\d\d:\d\d))?$/);
							if (pDate[4] == undefined)
							{
								pDate[4] = "00:00";
								pDate[5] = "+00:00";
							}
							else if (pDate[5] == "Z")
								pDate[5] = "+00:00";
							item.pubDate = time(new Date(pDate[1]+"/"+pDate[2]+"/"+pDate[3]+" "+pDate[4]+" "+pDate[5]));
						}
						catch (e) {}
						break;

					case "pubDate":
						try
						{
							item.pubDate = time(itemSrc[j].firstChild.data);
						}
						catch (e) {}
						break;

					case "source":
						try
						{
							item.source = itemSrc[j].firstChild.data;
							if (itemSrc[j].hasAttribute("url")) item.sourceurl = itemSrc[j].getAttribute("url");
						}
						catch (e) {}
						break;
				}
			}
			data.push(eval(item.toSource()));
		}
	}
	catch (e) {}
	return data;
}

/**
 * Parse a string w/ RSS content and load/import to RSS Editor
 * @param nsIDOMXMLDocument aDoc XML document to be parsed
 * @param boolean aReplace TRUE for loading a complete file, FALSE for import
 * @return int Number of items loaded/imported, FALSE for failure
 */
function parseDoc(aDoc, aReplace)
{
	// contents of channelDataTemp are only copied to channelData if replace is true
	var channelDataTemp = new RSSChannel();
	const subtags = {image: {url: 1, title:1, link:1, description:1, width:1, height:1}, textInput: {title:1, description:1, name:1, link:1}};
	const days = {Sunday: 0, Monday: 1, Tuesday: 2, Wednesday: 3, Thursday: 4, Friday: 5, Saturday: 6};
	var items = [];
	try
	{
		if (aDoc.documentElement.nodeName != "rss" && (aDoc.documentElement.localName != "RDF" || aDoc.documentElement.namespaceURI != ns.rdf)) throw "notvalid";

		
		// process PIs
		for (var element = aDoc.firstChild; element != null; element = element.nextSibling)
		{
			if (element.nodeType != aDoc.PROCESSING_INSTRUCTION_NODE) continue;
			if (element.target != "xml-stylesheet") continue;
			var re = /[ \t\r\n]*(.*?)[ \t\r\n]*=[ \t\r\n]*"([^"]*)"/g;
			for (data = re.exec(element.data); data != null; data = re.exec(element.data))
			{
				switch (data[1])
				{
					case "href": channelDataTemp.stylesheet.href = data[2];
					case "type": channelDataTemp.stylesheet.type = data[2];
				}
			}
			if (channelDataTemp.stylesheet.href != "")
				break; // we can only get one stylesheet
			else
				channelDataTemp.stylesheet.type = ""; // reset type to avoid mixing up w/ other PIs
		}

		// process RSS channel
		var channels = aDoc.getElementsByTagName("channel");
		if (channels.length != 1) throw "notvalid";
		var channel = channels[0].childNodes;
		var count = channel.length;
		var j, key, ebuf, item, count2;
		var val;
		for (i=0; i<count; i++)
		{
			switch (channel[i].localName)
			{
				case "title":
				case "link":
				case "description":
				case "language":
				case "managingEditor":
				case "webMaster":
				case "generator":
				case "docs":
				case "ttl":
				case "rating":
					try
					{
						channelDataTemp[channel[i].nodeName] = channel[i].firstChild.data;
					}
					catch (e) {}
					break;

				case "rights":
					if (channel[i].namespaceURI != ns.dc) break;
				case "copyright":
					try
					{
						channelDataTemp.copyright = channel[i].firstChild.data;
					}
					catch (e) {}
					break;

				case "pubDate":
				case "lastBuildDate":
					try
					{
						channelDataTemp[channel[i].nodeName] = time(channel[i].firstChild.data);
					}
					catch (e) {}
					break;

				case "category":
					try
					{
						channelDataTemp.category = channel[i].firstChild.data;
						if (channel[i].hasAttribute("domain")) channelDataTemp.domain = channel[i].getAttribute("domain");
					}
					catch (e) {}
					break;

				case "cloud":
					for (key in channelDataTemp.cloud)
						if (channel[i].hasAttribute(key)) channelDataTemp.cloud[key] = channel[i].getAttribute(key);
					break;

				case "image":
				case "textInput":
					ebuf = channel[i].childNodes;
					count2 = ebuf.length;
					for (j=0; j<count2; j++)
					{
						try
						{ // error "undefined property" may occur, that's normal so discard it
							if (subtags[channel[i].localName][ebuf[j].nodeName])
								channelDataTemp[channel[i].localName][ebuf[j].nodeName] = ebuf[j].firstChild.data;
						}
						catch (e) {}
					}
					break;
				
				case "skipHours":
					ebuf = channel[i].childNodes;
					count2 = ebuf.length;
					for (j=0; j<count2; j++)
					{
						try
						{
							if (ebuf[j].nodeName == "hour")
							{
								val = parseInt(ebuf[j].firstChild.data);
								if (val >=0 && val < 24) channelDataTemp.skipHours[val] = true;
							}
						}
						catch (e) {}
					}
					break;

				case "skipDays":
					ebuf = channel[i].childNodes;
					count2 = ebuf.length;
					for (j=0; j<count2; j++)
					{
						try
						{
							if (ebuf[j].nodeName == "day")
							{
								val = ebuf[j].firstChild.data;
								if (val in days) channelDataTemp.skipDays[days[val]] = true;
							}
						}
						catch (e) {}
					}
					break;
			}
		}
		items = parseItems(aDoc.getElementsByTagName("item"));
	}
	catch (e)
	{
		openMsgBox(localize("NotRSSFile"), 16, sdi.baseTitle);
		return false;
	}

	if (aReplace)
	{
		channelData = channelDataTemp;
		resetChannel();

		var list = document.getElementById("itemList");
		while (list.lastChild)
			list.removeChild(list.lastChild);
	}

	count = items.length;
	for (i=0; i<count; i++)
		addListitem(items[i], false);

	return count;
}


/**
 * Save data (called by "save" or "save-as" or "export" or "publish")
 */
function saveRSS(foStream, type)
{
	itemUpdate();

	foStream.write(rssXMLHeader, rssXMLHeader.length);
	(new XMLSerializer).serializeToStream(buildRSS(type), foStream, "UTF-8");
}

/**
 * Build DOM element "rss"
 */
function buildRSS(type)
{
	var doc = document.implementation.createDocument(null, "rss", null);
	doc.documentElement.setAttribute("version", "2.0");
	doc.documentElement.appendChild(buildChannel(type));
	if (getValue("miscStylesheetHref") != "") doc.insertBefore(buildPI(), doc.documentElement);
	return doc;
}


/**
 * Build DOM PI
 */
function buildPI()
{
	var d = document.implementation.createDocument(null, "rss", null);
	var ref = {miscStylesheetHref: "href", miscStylesheetType: "type"};
	var buf = [];
	var stripped = false;
	var a;
	for (var i in a = ["miscStylesheetHref", "miscStylesheetType"])
	{
		if (/"|\?>/.test(getValue(a[i])))
		{
			setValue(a[i], getValue(a[i]).replace(/"|\?>/g, ""));
			stripped = true;
		}
		if (getValue(a[i]) != "") buf[i] = ref[a[i]] + "=\"" + getValue(a[i]) +"\"";
	}
	if (stripped) openMsgBox(localize("StyleSheetInvalidChars"), 16, sdi.baseTitle);
	return d.createProcessingInstruction("xml-stylesheet", buf.join(" "));
}

/**
 * Build DOM element "channel"
 */
function buildChannel(type)
{
	var d = document.implementation.createDocument(null, "rss", null);
	var e = d.createElement("channel");
	var b; // buffer
	var key, count, i, a;
	var ranges; // ranges of items to be built

	// Required elements
	insertElement(e, "title", getValue("channelTitle"));
	insertElement(e, "link", getValue("channelLink"));
	insertElement(e, "description", getValue("channelDesc"));

	// Optional elements
	getValue("channelLang") != "" && insertElement(e, "language", getValue("channelLang"));
	getValue("channelCopyright") != "" && insertElement(e, "copyright", getValue("channelCopyright"));
	getValue("channelManagingEditor") != "" && insertElement(e, "managingEditor", getValue("channelManagingEditor"));
	getValue("channelWebMaster") != "" && insertElement(e, "webMaster", getValue("channelWebMaster"));

	// Dates are optional, validated and converted to UTC
	if ((b = new Date(getValue("channelPubDate")).toUTCString()) != "Invalid Date")
		insertElement(e, "pubDate", b);
	if ((b = new Date(getValue("channelLastBuildDate")).toUTCString()) != "Invalid Date")
		insertElement(e, "lastBuildDate", b);

	// Optional category and domain
	if (getValue("channelCategory") != "" || getValue("channelDomain") != "")
	{
		b = insertElement(e, "category", getValue("channelCategory"));
		getValue("channelDomain") != "" && b.setAttribute("domain", getValue("channelDomain"));
	}

	// Optional generator and docs
	getValue("channelGenerator") != "" && insertElement(e, "generator", getValue("channelGenerator"));
	getValue("channelDocs") != "" && insertElement(e, "docs", getValue("channelDocs"));

	// Optional cloud
	if (getValue("channelCloudDomain") != "" || getValue("channelCloudPort") != "" || getValue("channelCloudPath") != "" || getValue("channelCloudRegisterProcedure") != "" || getValue("channelCloudProtocol") != "")
	{
		e.appendChild(b = d.createElement("cloud"));
		b.setAttribute("domain", getValue("channelCloudDomain"));
		b.setAttribute("port", getValue("channelCloudPort"));
		b.setAttribute("path", getValue("channelCloudPath"));
		b.setAttribute("registerProcedure", getValue("channelCloudRegisterProcedure"));
		b.setAttribute("protocol", getValue("channelCloudProtocol"));
	}

	// Optional ttl
	getValue("channelTtl") != "" && insertElement(e, "ttl", getValue("channelTtl"));

	// Optional image
	if (getValue("channelImage") != "")
	{
		// Common statement whether modified or not
		e.appendChild(b = d.createElement("image"));
		insertElement(b, "url", getValue("channelImage"));
		if (getValue("channelImage") != channelData.image.url)
		{
			// Modified, set the contents of title and link to channel/title and channel/link according to RSS 2.0 spec
			insertElement(b, "title", getValue("channelTitle"));
			insertElement(b, "link", getValue("channelLink"));
		}
		else
		{
			// Not modified, use data of the opened file
			// Insert title and link if not set (according to RSS 2.0 spec)
			insertElement(b, "title", channelData.image.title != "" ? channelData.image.title : getValue("channelTitle"));
			insertElement(b, "link", channelData.image.link != "" ? channelData.image.link : getValue("channelLink"));
			// Optional fields
			channelData.image.description != "" && insertElement(b, "description", channelData.image.description);
			channelData.image.width != "" && insertElement(b, "width", channelData.image.width);
			channelData.image.height != "" && insertElement(b, "height", channelData.image.height);
		}
	}

	// Optional rating
	getValue("channelRating") != "" && insertElement(e, "rating", getValue("channelRating"));

	// Optional textInput
	if (getValue("channelTextInputTitle") != "" || getValue("channelTextInputDescription") != "" || getValue("channelTextInputName") != "" || getValue("channelTextInputLink") != "")
	{
		e.appendChild(b = d.createElement("textInput"));
		insertElement(b, "title", getValue("channelTextInputTitle"));
		insertElement(b, "description", getValue("channelTextInputDescription"));
		insertElement(b, "name", getValue("channelTextInputName"));
		insertElement(b, "link", getValue("channelTextInputLink"));
	}

	// Optional skipHours
	b = d.createElement("skipHours");
	count = 0;
	for (i=0; i<24; i++)
		if (document.getElementById("channelSkipHours." + i).checked)
		{
			insertElement(b, "hour", i);
			count++;
		}
	// only insert if there is any content
	count && e.appendChild(b);

	// Optional skipHours
	b = d.createElement("skipDays");
	count = 0;
	for (i in a = ["Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"])
		if (document.getElementById("channelSkipDays." + i).checked)
		{
			insertElement(b, "day", a[i]);
			count++;
		}
	// only insert if there is any content
	count && e.appendChild(b);

	// Set range of items to be built
	var tree = document.getElementById("tree");
	switch (type)
	{
		case rssType_selection:
			ranges = treeSelection(tree);
			break;
		case rssType_all:
		default:
			ranges = [{start: 0, end: tree.view.rowCount - 1}];
	}
	buildItems(e, ranges);

	return e;
}

/**
 * Append the "item" elements to the supplied element according to the ranges given
 */
function buildItems(element, ranges)
{
	var list = document.getElementById("itemList");
	var count = ranges.length;
	var item, buf, ebuf, b;
	for (var i=0; i<count; i++)
	{
		for (var j=ranges[i].start; j<=ranges[i].end; j++)
		{
			try
			{
				item = eval(list.childNodes[j].firstChild.lastChild.getAttribute("label"));
				buf = element.ownerDocument.createElement("item");
				(item.title != "" || item.desc == "") && insertElement(buf, "title", item.title);
				item.link != "" && insertElement(buf, "link", item.link);
				item.desc != "" && insertElement(buf, "description", item.desc);
				item.author != "" && insertElement(buf, "author", item.author);
				if (item.category != "")
				{
					ebuf = insertElement(buf, "category", item.category);
					item.domain != "" && ebuf.setAttribute("domain", item.domain);
				}
				item.comments != "" && insertElement(buf, "comments", item.comments);
				for (var key in item.enclosure)
				{
					if (item.enclosure[key] == "") continue;
					buf.appendChild(ebuf = buf.ownerDocument.createElement("enclosure"));
					for (key in item.enclosure) // reuse key even if it is used in the outer loop
						ebuf.setAttribute(key, item.enclosure[key]);
					break;
				}
				if (item.guid != "")
				{
					ebuf = insertElement(buf, "guid", item.guid);
					item.isPermaLink || ebuf.setAttribute("isPermaLink", "false");
				}
				if ((b = new Date(item.pubDate).toUTCString()) != "Invalid Date")
					insertElement(buf, "pubDate", b);
				(item.source != "" || item.sourceurl != "") && insertElement(buf, "source", item.source).setAttribute("url", item.sourceurl);

				element.appendChild(buf);
			}
			catch (e) {} //alert
		}
	}
}


/**
 * Confirm whether current file can be closed
 * @return boolean TRUE for yes, FALSE for no
 */
function confirmCloseFile()
{
	if (sdi.modified == false) return true;
	switch (openMsgBox(localize("SaveBeforeClosing", sdi.name), 51, sdi.baseTitle))
	{
		case 6: saveFile(); return !sdi.modified;
		case 7: return true;
		default: return false;
	}
}


/**
 * Start a new file (delete everything in the editing environment)
 */
function newFile()
{
	if (!confirmCloseFile()) return;
	// Empty channel data
	channelData = new RSSChannel();
	resetChannel();

	// Reset items
	var list = document.getElementById("itemList");
	while (list.lastChild)
		list.removeChild(list.lastChild);
	// Reset fields in Items Tab: done by itemSelect() called by onselect event

	// Empty buffer state
	sdi.content = null;
	sdi.modified = false;
}


/**
 * Parse a URI into RSS Editor
 * @param nsIURI aURI URI to be parsed
 * @param boolean aReplace TRUE for loading a complete file, FALSE for import
 * @return int Number of items loaded/imported, FALSE for failure
 */
function parseURI(aURI, aReplace)
{
	var resume = function(aStatus, aResult)
	{
		if (aStatus)
		{
			openMsgBox(localize("OpenFailure", aStatus), 16, sdi.baseTitle);
			return;
		}
		var dp = new DOMParser, doc;
		if (typeof aResult == "string")
		{
			// Firefox <= 1.0
			var ss = Components.classes["@mozilla.org/io/string-input-stream;1"].getService(Components.interfaces.nsIStringInputStream);
			ss.setData(aResult, aResult.length);
			doc = dp.parseFromStream(ss, null, aResult.length, "text/xml");
		}
		else
			// Firefox 1.1
			doc = dp.parseFromBuffer(aResult, aResult.length, "text/xml");
		parseDoc(doc, aReplace);
		openNotifier.succeed();
	};
	parseURI1(aURI, aReplace, resume);
}


function parseURI1(aURI, aReplace, aCallback)
{
	var ioService  = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
	var streamLoader=Components.classes["@mozilla.org/network/stream-loader;1"].createInstance(Components.interfaces.nsIStreamLoader);
	var channel = ioService.newChannelFromURI(aURI);
	if(aURI.scheme=="http" || aURI.scheme=="https") channel.loadFlags |= Components.interfaces.nsIRequest.LOAD_BYPASS_CACHE;

	var listener =
	{
		_callback: aCallback,
		onStreamComplete: function(loader, ctxt, status, resultLength, result)
		{
			if (this._callback) this._callback(status, result);
		}
	}

	streamLoader.init(channel, listener, null);
}


/**
 * Parse a file into RSS Editor
 * @param nsIFile aFile File to be parsed
 * @param boolean aReplace TRUE for loading a complete file, FALSE for import
 * @return int Number of items loaded/imported, FALSE for failure
 */
function parseFile(aFile, aReplace)
{
	var io = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
	var fileURI = io.newFileURI(aFile);
	return parseURI(fileURI, aReplace);
}


/**
 * Store the file/url and updates window title on successful open
 */
var openNotifier =
{
	obj: null,
	start: function(v) { this.obj = v; },
	succeed: function()
	{
		sdi.content = this.obj;
		sdi.modified = false;
	}
}


/**
 * Open an RSS file
 */
function openFile()
{
	if (!confirmCloseFile()) return;

	var data = {};
	var file = new rssFilePicker().open(localize("Open"));
	if (file == null) return;
	openNotifier.start(file);
	if (parseFile(file, true) === false) return;
}


/**
 * Open an URI
 */
function openURI()
{
	if (!confirmCloseFile()) return;

	var arg = {title: localize("OpenURI")};
	window.openDialog("uripicker.xul", "_blank", "modal", arg);
	if (arg.value != true) return
	openNotifier.start(arg.uri);
	if (parseURI(arg.uri, true) === false) return;
}


/**
 * Import a string which is a valid RSS file
 * @param string aString
 */
function importString(aString)
{
	var dp = new DOMParser, doc;
	doc = dp.parseFromString(aString, "text/xml");

	// paste it before "selectedItem"
	var count = parseDoc(doc, false);
	if (!count) return; // return for both 0 (no items imported) and FALSE (failure)

	var view = document.getElementById("tree").view;
	// Select the pasted items
	if (selectedItem == null)
		view.selection.rangedSelect(count - 1, 0, false);
	else
	{
		var index = view.getIndexOfItem(selectedItem.parentNode);
		view.selection.rangedSelect(index - 1, index - count, false);
	}
	itemSelect(false);
}


/**
 * Import an RSS file
 */
function importFile()
{
	var data = {};
	var file = new rssFilePicker().open(localize("Import"));
	if (file == null) return;
	sdi.modified = true;
	parseFile(file, false);
}


/**
 * Import an RSS file from a URI
 */
function importURI()
{
	var arg = {title: localize("ImportURI")};
	window.openDialog("uripicker.xul", "_blank", "modal", arg);
	if (arg.value != true) return
	sdi.modified = true;
	parseURI(arg.uri, false);
}


/**
 * Save RSS to a file
 * @param nsIFile file
 * @param boolean selectionOnly
 * @return boolean TRUE for success, FALSE for failure
 */
function save(file, selectionOnly)
{
	try
	{
		var nsIFileOutputStream = Components.interfaces.nsIFileOutputStream;
		var foStream = Components.classes["@mozilla.org/network/file-output-stream;1"].createInstance(nsIFileOutputStream);
		foStream.init(file, 0x2a, 0600, false); // wronly | create | truncate
		saveRSS(foStream, selectionOnly ? rssType_selection : rssType_all);
		foStream.close();
		return true;
	}
	catch (e)
	{
		openMsgBox(localize("SaveFailure", e.toString()), 16, sdi.baseTitle);
		return false;
	}
}


/**
 * Save to URI
 * @param nsIURI uri
 * @param boolean selectionOnly
 * @param function callback(status) Callback function for returning
 */
function saveToURI(uri, selectionOnly, callback)
{
	if (uri.scheme == "http" || uri.scheme == "https")
	{
		openMsgBox(localize("SaveToHTTPNotAllowed"), 16, sdi.baseTitle);
		return false;
	}

	var io = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
	var sos = new StringOutputStream();
	var sis = Components.classes["@mozilla.org/io/string-input-stream;1"].getService(Components.interfaces.nsIStringInputStream);

	saveRSS(sos, selectionOnly ? rssType_selection : rssType_all);
	sis.setData(sos.buf, -1);

	var channel = io.newChannelFromURI(uri).QueryInterface(Components.interfaces.nsIUploadChannel);
	channel.setUploadStream(sis, "text/xml", -1); // text/xml is temporary

	var listener =
	{
		_callback: callback,
		onDataAvailable: function(channel, ctxt, input, sourceOffset, count)
		{
			const sis=Components.classes["@mozilla.org/scriptableinputstream;1"].createInstance(Components.interfaces.nsIScriptableInputStream);
			sis.init(input);
			this._errorData +=sis.read(count);
		},
		onStartRequest: function(channel, ctxt) {},
		onStopRequest: function(channel, ctxt, status)
		{
			if (this._callback) this._callback(status);
		}
	};
	channel.asyncOpen(listener, null);
	return true;
}


/**
 * Save current file
 */
function saveFile()
{
	sdi.content ? ("spec" in sdi.content ? saveURI1(sdi.content) : save(sdi.content, false)) && (sdi.modified = false) : saveAs();
}


/**
 * Save current file as a new file
 */
function saveAs()
{
	var file = new rssFilePicker().save(localize("Save As"));
	if (!file) return;
	if (!save(file, false)) return;
	sdi.content = file;
	sdi.modified = false;
}


/**
 * Save to URI
 * @param nsIURI uri
 */
function saveURI1(uri)
{
	var resume = function(aStatus)
	{
		if (aStatus)
		{
			openMsgBox(localize("SaveURIFailure", aStatus), 16, sdi.baseTitle);
			return;
		}
		sdi.content = uri;
		sdi.modified = false;
	};
	saveToURI(uri, false, resume);
}

/**
 * Save to URI using uripicker
 */
function saveURI()
{
	var arg = {title: localize("SaveURI"), uri: sdi.content && "spec" in sdi.content ? sdi.content.clone() : undefined};
	window.openDialog("uripicker.xul", "_blank", "modal", arg);
	if (arg.value != true) return;
	saveURI1(arg.uri);
}


/**
 * Export a file
 */
function exportFile()
{
	var file = new rssFilePicker().save(localize("Export"));
	file && save(file, true);
}


/**
 * Export to URI
 */
function exportURI()
{
	var arg = {title: localize("ExportURI")};
	window.openDialog("uripicker.xul", "_blank", "modal", arg);
	if (arg.value != true) return;
	var resume = function(aStatus)
	{
		if (aStatus) openMsgBox(localize("SaveURIFailure", aStatus), 16, sdi.baseTitle);
	};
	saveToURI(arg.uri, true, resume);
}


/**
 * Copy items to clipboard - as an RSS file
 */
function itemsCopy()
{
	itemUpdate();

	var objXMLSerializer = new XMLSerializer;
	var str = rssXMLHeader;
	str += objXMLSerializer.serializeToString(buildRSS(rssType_selection));

	const gClipboardHelper = Components.classes["@mozilla.org/widget/clipboardhelper;1"].getService(Components.interfaces.nsIClipboardHelper);
	gClipboardHelper.copyString(str);
}


/**
 * Paste from an RSS file in the clipboard
 */
function itemsPaste()
{
	importString(getClipboardText());
}