const dir = Components.classes["@mozilla.org/file/directory_service;1"].getService(Components.interfaces.nsIProperties);
const io = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
const rdfCU = Components.classes["@mozilla.org/rdf/container-utils;1"].createInstance(Components.interfaces.nsIRDFContainerUtils);
const rdfS = Components.classes["@mozilla.org/rdf/rdf-service;1"].getService(Components.interfaces.nsIRDFService);
const pwdMgr = Components.classes["@mozilla.org/passwordmanager;1"].getService(Components.interfaces.nsIPasswordManager).QueryInterface(Components.interfaces.nsIPasswordManagerInternal);

const folderRoot = rdfS.GetResource("NC:Root");

var ds; // datasource

function uriSelect()
{
	var tree = document.getElementById("uriList");

	if (tree.currentIndex == -1)
		document.getElementById("removeURI").setAttribute("disabled", "true");
	else
	{
		document.getElementById("removeURI").removeAttribute("disabled");
		try
		{
			// Firefox <= 1.0
			setValue("host", tree.view.getCellText(tree.currentIndex, "treehost"));
			setValue("path", tree.view.getCellText(tree.currentIndex, "treepath"));
			setValue("username", tree.view.getCellText(tree.currentIndex, "treeusername"));
		}
		catch (e)
		{
			// Firefox 1.1
			setValue("host", tree.view.getCellText(tree.currentIndex, tree.treeBoxObject.columns.getNamedColumn("treehost")));
			setValue("path", tree.view.getCellText(tree.currentIndex, tree.treeBoxObject.columns.getNamedColumn("treepath")));
			setValue("username", tree.view.getCellText(tree.currentIndex, tree.treeBoxObject.columns.getNamedColumn("treeusername")));
		}
		loadPassword(false);
	}
}

function uriDblClick(aEvent)
{
	// Accept only left button
	if (aEvent.button != 0) return;
	// Accept only if mouse is on selection (not other item or empty space)
	var tree = document.getElementById("uriList");
	var row = {}, colID = {}, obj = {};
	tree.treeBoxObject.getCellAt(aEvent.clientX, aEvent.clientY, row, colID, obj);
	if (tree.currentIndex == row.value) document.documentElement.acceptDialog();
}


function appendURIToList(uri)
{
	var subject, predicate, object;
	var uriClone = uri.clone();
	try { uriClone.password = "" } catch(e) {}
	subject = rdfS.GetResource(uriClone.spec);

	// add the triples
	predicate = rdfS.GetResource("http://home.netscape.com/NC-rdf#Host");
	object = rdfS.GetLiteral(getValue("host"));
	ds.Assert(subject, predicate, object, true);

	if (getValue("path") != "")
	{
		predicate = rdfS.GetResource("http://home.netscape.com/NC-rdf#Path");
		object = rdfS.GetLiteral(getValue("path"));
		ds.Assert(subject, predicate, object, true);
	}

	if (getValue("username") != "")
	{
		predicate = rdfS.GetResource("http://home.netscape.com/NC-rdf#Username");
		object = rdfS.GetLiteral(getValue("username"));
		ds.Assert(subject, predicate, object, true);
	}

	// add to the container
	var container = rdfCU.MakeBag(ds, folderRoot);
	if (container.IndexOf(subject) == -1)
		container.AppendElement(subject);

	ds.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource).Flush();
}


function removeURIFromList()
{
	var tree = document.getElementById("uriList");
	var index = tree.currentIndex;
	if (index < 0) return;

	if (openMsgBox(localize("ConfirmRemoveURI"), 33, localize("ConfirmRemoveURITitle")) != 1) return;

	var subject = tree.view.getResourceAtIndex(index);
	var preE = ds.ArcLabelsOut(subject);
	var username; // stores username for checking
	while (preE.hasMoreElements())
	{
		var predicate = preE.getNext().QueryInterface(Components.interfaces.nsIRDFResource);
		var objE = ds.GetTargets(subject, predicate, true);
		while (objE.hasMoreElements())
		{
			var object = objE.getNext().QueryInterface(Components.interfaces.nsIRDFNode);
			if (predicate.EqualsString("http://home.netscape.com/NC-rdf#Username")) username = object;
			ds.Unassert(subject, predicate, object);
		}
	}

	// delete from the container
	var container = Components.classes["@mozilla.org/rdf/container;1"].createInstance(Components.interfaces.nsIRDFContainer);
	// no need to catch exception, because folderRoot should be present if there is a deletable item
	container.Init(ds, folderRoot);
	container.RemoveElement(subject, true);

	ds.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource).Flush();

	// remove password if no other URI have the same prepath and username
	if (username) // skip if username is not present (because no password would be stored)
	{
		var uri = io.newURI(subject.Value, null, null);
		predicate = rdfS.GetResource("http://home.netscape.com/NC-rdf#Username");
		var subE = ds.GetSources(predicate, username, true);
		var found = false;
		while (subE.hasMoreElements())
		{
			subject = subE.getNext().QueryInterface(Components.interfaces.nsIRDFResource);
			var uri1 = io.newURI(subject.Value, null, null);
			if (uri.prePath == uri1.prePath) { found = true; break; }
		}
		if (!found) removePassword(uri);
	}
}


function loadPassword(searchUsername)
{
	var uri;
	var host = {}, username = {}, password = {};
	
	try
	{
		if (!searchUsername && getValue("username") == "") throw "nopassword";
		try
		{
			uri = io.newURI(getValue("host"), null, null);
		}
		catch(e)
		{
			uri = io.newURI("ftp://" + getValue("host"), null, null);
		}
		if (uri.scheme == "file") throw "nopassword";
		pwdMgr.findPasswordEntry(uri.prePath, getValue("username"), null, host, username, password);
	}
	catch(e)
	{
		setValue("password", "");
		return;
	}
	setValue("username", username.value);
	setValue("password", password.value);
}


function rememberPassword(uri)
{
	try
	{
		if (uri.password == "") return;
		var uriClone = uri.clone();
		uriClone.userPass = "";
		pwdMgr.addUser(uriClone.prePath, uri.username, uri.password);
	}
	catch(e) {}
}


function removePassword(uri)
{
	try
	{
		var uriClone = uri.clone();
		uriClone.userPass = "";
		pwdMgr.removeUser(uriClone.prePath, uri.username);
	}
	catch(e) {}
}


function parseURI()
{
	// init text fields
	var uri;
	try
	{
		try
		{
			uri = io.newURI(getValue("host"), null, null);
		}
		catch(e)
		{
			uri = io.newURI("ftp://" + getValue("host"), null, null);
		}
		if (uri.scheme == "file")
		{
			setValue("username", "")
			setValue("password", "");
			setValue("path", "");
		}
		else
		{
			if (uri.username != "") setValue("username", decodeURI(uri.username));
			if (uri.password != "") setValue("password", decodeURI(uri.password));
			if (uri.path != "" && uri.path != "/") setValue("path", decodeURI(uri.path));
			if (uri.scheme == "ftp")
				setValue("host", uri.hostPort);
			else
			{
				uri.userPass = "";
				setValue("host", uri.prePath);
			}
		}
	}
	catch(e) {};
}

function init()
{
	if (window.opener)
	{
		window.sizeToContent();
		window.moveTo(window.opener.screenX+(window.opener.outerWidth-window.outerWidth)/2,window.opener.screenY+(window.opener.outerHeight-window.outerHeight)/2);
	}
	try
	{
		document.documentElement.setAttribute("title", window.arguments[0].title); // Firefox <= 1.0
		document.title = window.arguments[0].title; // Firefox 1.1
	} catch(e) {}
	try
	{
		if (!("uri" in window.arguments[0])) throw (""); // just a convenient way to exit try loop;
		var uri = window.arguments[0].uri.clone();
		if (1) // donno how to check whether uri is a real nsIURI... do later
		{
			if (uri.scheme == "file")
				setValue("host", uri.spec);
			else
			{
				if (uri.username != "") setValue("username", decodeURI(uri.username));
				if (uri.password != "") setValue("password", decodeURI(uri.password));
				if (uri.path != "" && uri.path != "/") setValue("path", decodeURI(uri.path));
				if (uri.scheme == "ftp")
					setValue("host", uri.hostPort);
				else
				{
					uri.userPass = "";
					setValue("host", uri.prePath);
				}
			}
		}
	}
	catch(e) {}

	// init url tree
	var urlFile = dir.get("UChrm", Components.interfaces.nsIFile);
	urlFile.append("rsseditor");
	urlFile.append("uriList.rdf");
	ds = rdfS.GetDataSourceBlocking(io.newFileURI(urlFile).spec);

	var cds = document.getElementById("uriList").database;
	cds.AddDataSource(ds);
	try { cds.RemoveDataSource(rdfS.GetDataSourceBlocking("rdf:localstore")); } catch (e) {} // Firefox 1.1 doesn't have rdf:localstore so can't be unregistered

	document.getElementById("uriList").builder.rebuild();
}

function okay()
{
	var uri;
	try
	{
		try
		{
			uri = io.newURI(getValue("host"), null, null);
		}
		catch(e)
		{
			uri = io.newURI("ftp://" + getValue("host"), null, null);
		}
		if (uri.scheme != "file")
		{
			uri.username = getValue("username");
			if (uri.username != "") uri.password = getValue("password");
			uri.path = getValue("path");
		}
	}
	catch(e) { return false; }

	// remember uri and password
	document.getElementById("rememberURI").checked && getValue("host") != "" && appendURIToList(uri);
	if (document.getElementById("rememberPassword").checked)
		getValue("host") != "" && rememberPassword(uri);
	else
		getValue("host") != "" && removePassword(uri);

	// return value
	try
	{
		window.arguments[0].uri = uri;
		window.arguments[0].value = true;
	}
	catch(e) {}
	return true;
}

function cancel()
{
	try
	{
		window.arguments[0].uri = null;
		window.arguments[0].value = false;
	}
	catch(e) {}
}